/*   File: atphyslib.c
     Common physics functions for Accelerator Toolbox
(1)
     A. Terebilo   10/28/04
(2)  
   Laurent S. NAdolski, April 8th, 2007
          add energy dependance in edge_fringe and edge
(3)
Added flags to turn on dipole fringe field and edge effects
in dipoles.

Modified by Jianfeng Zhang @ LAL, 30/04/2013.

(4) 
05/06/2013 written by Jianfeng Zhang @ LAL
Added Alex Loulergue's geometric correction of the 
dipole fringe field, but without the correction
term 1+r[4] of py in Forest: SSC-141; because this 
correction term only works for Soleil lattice, but not works
for ThomX and SuperB DR.

*/
	
#include "mex.h"
void edge(double* r, double inv_rho, double edge_angle)
{	/* Edge focusing in dipoles with hard-edge field; geometric rotation */
  double psi = inv_rho*tan(edge_angle);
  
  /* r[1]+=r[0]*psi; */
  r[1]+=r[0]*psi/(1+r[4]*1); 
  r[3]-=r[2]*psi/(1+r[4]*1);






}


void edge_fringe(double* r, double inv_rho, double edge_angle, double fint, double gap, int entrance)
{   /* Edge focusing in dipoles with fringe field */
  double fx = inv_rho*tan(edge_angle);
  double fy = 0.0;
  /*  double psi_bar = edge_angle-inv_rho*gap*fint*(1+sin(edge_angle)*sin(edge_angle))/cos(edge_angle)/(1+r[4]); */

double psi_bar = edge_angle-inv_rho*gap*fint*(1+sin(edge_angle)*sin(edge_angle))/cos(edge_angle)/(1+r[4]*0);

  /*modification for ThomX; Loulergue's geometric correction */
 if(entrance == 1)
   fy = inv_rho*tan(psi_bar + r[1]/(1+r[4]));
 else
   fy = inv_rho*tan(psi_bar - r[1]/(1+r[4]));

  /* r[1]+=r[0]*fx;
     r[3]-=r[2]*fy; */
  /*  r[1]+=r[0]*fx/(1+r[4]);
      r[3]-=r[2]*fy/(1+r[4]); */

 /*remove geometric correction to py in Forest: SSC-141, only correct
   to Soleil lattice, but not correct to ThomX and SuperB DR lattice.*/
  r[1]+=r[0]*fx/(1+r[4]*0);
  r[3]-=r[2]*fy/(1+r[4]*0);
}


/* /\*Alex Dragt's model...*\/ */
/* void edge_fringe(double* r, double inv_rho, double edge_angle, double fint, double gap, int entrance) */
/* {   /\* Edge focusing in dipoles with fringe field *\/ */
/*   double fx = inv_rho*tan(edge_angle); */
/*   double fy = 0.0; */
/*   /\*  double psi_bar = edge_angle-inv_rho*gap*fint*(1+sin(edge_angle)*sin(edge_angle))/cos(edge_angle)/(1+r[4]); *\/ */

/* double psi_bar = edge_angle-inv_rho*gap*fint*(1+sin(edge_angle)*sin(edge_angle))/cos(edge_angle)/(1+r[4]*0); */

/*  double phi_bar = sqrt((1+r[4])*(1+r[4])-r[1]*r[1]); */
/*  double phi_bar2= phi_bar*phi_bar; */
/*  double phi_bar3 = phi_bar2*phi_bar; */
 

/*  r[3]-=inv_rho*r[2]*r[1]/phi_bar+inv_rho*tan(edge_angle-psi_bar); */
/*  r[0]+=inv_rho*0.5*r[2]*r[2]*(1+r[4])*(1+r[4])/phi_bar3; */
/*  r[1]+=r[0]*fx; */

/* } */
