function handles = plotfamilystartup(handles)
% plotfamilystartup - Additional plotfamily menu for SOLEIL

%% Laurent Nadolski
% Adapted form Gregory Portmann
%
%
%  Modified by Jianfeng Zhang for ThomX @ LAL, 21/06/2013

% Disable setpoint GUI
set(handles.DisabledSetpointChanges, 'Checked', 'On');

Sectors = 16; % number of sectors
Straights = 24; % numberof straight sections
StraigthPosition = [0; getspos('BPMx', getidbpmlist)]; 
% is it the best place for these definitions ?
StraigthName = {'SDL01 (INJECTION)', 'SDM02 (CRYO2)', 'SDC02 (ALPHA)', 'SDM03 (CRYO1)', 'SDC03 (PSICHE)', 'SDM04 (PLEIADES)', ...
                'SDL05 (DESIRS)', 'SDM06 (Empty)', 'SDC06 (CRISTAL)', 'SDM07 (DEIMOS)', 'SDC07 (GALAXIES)', 'SDM08 (TEMPO)', ...
                'SDL09 (FBT)', 'SDM10 (HERMES)', 'SDC10 (PX1)', 'SDM11 (PX2)', 'SDC11 (SWING)', 'SDM12 (ANTARES)', ...
                'SDL13 (NANO)', 'SDM14 (SEXTANTS)', 'SDC14 (SIXS)', 'SDM15 (CASSIOPEE)', 'SDC15 (SIRIUS)', 'SDM16 (LUCIA)', ...
                };
Superperiods = 2; % number of superperiods

L = getfamilydata('Circumference');
if ~isempty(L)
    % Add a sector menu
    % Using a [Arc Straight] nomenclature
    Menu0 = uimenu(handles.figure1, 'Label', 'Sectors');
    set(Menu0, 'Position', 3);
    set(Menu0, 'Separator', 'On');

    % Arc sections
    Extra = 5;  % meters
    i = 1;
    Menu1 = uimenu(Menu0, 'Label',sprintf('Arc Sector %d',i));
    set(Menu1,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[0 Extra+L/Sectors]+(i-1)*L/Sectors),'],guidata(gcbo))']);
    for i = 2:Sectors-1
        Menu1 = uimenu(Menu0, 'Label',sprintf('Arc Sector %d',i));
        set(Menu1,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[0-Extra Extra+L/Sectors]+(i-1)*L/Sectors),'],guidata(gcbo))']);
    end
    i = Sectors;
    Menu1 = uimenu(Menu0, 'Label',sprintf('Arc Sector %d',i));
    set(Menu1,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[L-L/Sectors-Extra L]),'],guidata(gcbo))']);

    % Add Straight section menu
    Menu0 = uimenu(handles.figure1, 'Label', 'Straights');
    set(Menu0, 'Position', 4);
    set(Menu0, 'Separator', 'On');
    % Straight sections
    Extra = 10;  % meters
    for i = 1:Straights-1
        Menu1 = uimenu(Menu0, 'Label',sprintf('%s',StraigthName{i}));
        set(Menu1,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[-Extra+StraigthPosition(2*i-1) Extra+StraigthPosition(2*i)]),'],guidata(gcbo))']);
    end
    Menu1 = uimenu(Menu0, 'Label',sprintf('%s',StraigthName{end}));
    set(Menu1,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[-Extra+StraigthPosition(end-1) Extra+StraigthPosition(end)]),'],guidata(gcbo))']);

    % Add Superperiod menu
    Menu0 = uimenu(handles.figure1, 'Label', 'Superperiod');
    set(Menu0, 'Position', 5);
    set(Menu0, 'Separator', 'On');

    handles.all = uimenu(Menu0, 'Label',sprintf('All Ring'));
    set(handles.all,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[0 L]),'],guidata(gcbo))']);
    set(handles.all,'Checked', 'On');
    
    handles.superperiod1 = uimenu(Menu0, 'Label',sprintf('Superperiod %d',1));
    set(handles.superperiod1,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[0 L/Superperiods]),'],guidata(gcbo))']);

    handles.superperiod2 = uimenu(Menu0, 'Label',sprintf('Superperiod %d',2));
    set(handles.superperiod2,'Callback', ['plotfamily(''HorizontalAxisSector_Callback'',gcbo,[',sprintf('%f %f',[L/Superperiods 2*L/Superperiods]),'],guidata(gcbo))']);

end

